Index: lib/Qpsmtpd/Transaction.pm
===================================================================
RCS file: /cvs/public/qpsmtpd/lib/Qpsmtpd/Transaction.pm,v
retrieving revision 1.6
diff -u -r1.6 Transaction.pm
--- lib/Qpsmtpd/Transaction.pm	10 Jun 2003 10:15:42 -0000	1.6
+++ lib/Qpsmtpd/Transaction.pm	22 Dec 2003 21:01:56 -0000
@@ -4,7 +4,7 @@
 use strict;
 use Qpsmtpd::Utils;
 
-use IO::File qw(O_RDWR O_CREAT);
+use IO::File qw(O_RDWR O_CREAT O_EXCL);
 
 # For unique filenames. We write to a local tmp dir so we don't need
 # to make them unpredictable.
@@ -80,8 +80,25 @@
      -d $spool_dir or mkdir($spool_dir, 0700) or die "Could not create spool_dir: $!";
      $self->{_filename} = $spool_dir . join(":", time, $$, $transaction_counter++);
      $self->{_filename} =~ tr!A-Za-z0-9:/_-!!cd;
-    $self->{_body_file} = IO::File->new($self->{_filename}, O_RDWR|O_CREAT)    
-      or die "Could not open file $self->{_filename} - $! "; # . $self->{_body_file}->error;
+
+     # Create the temporary file with exclusive create.  This prevents a
+     # potential security problem where the temporary file could be a
+     # symlink.  In that case, we could accidentally overwrite the contents.
+     # We're creating a temporary file so there _must not_ be an existing
+     # file already in place.
+     #
+     # NOTE: The O_EXCL flag doesn't work correctly using NFS.  However,
+     # if you have your temporary directory mounted through NFS, you have
+     # bigger problems. - rjkaes
+     $self->{_body_file} =
+	 IO::File->new($self->{_filename}, O_RDWR | O_CREAT | O_EXCL, 0600)
+		 or die "Could not open file $self->{_filename} - $! ";
+
+     # Unlink the opened file right away.  When qpsmtpd closes, the file will
+     # automatically be removed from the file system (since no other
+     # processes will be associated with it. - rjkaes
+     unlink $self->{_filename}
+	 or $self->log(0, 'Could not unlink [', $self->{_filename}, "] after opening: $!");
   }
   # go to the end of the file
   seek($self->{_body_file},0,2)
@@ -116,10 +133,13 @@
 
 sub DESTROY {
   my $self = shift;
-  # would we save some disk flushing if we unlinked the file before
-  # closing it?
 
-  undef $self->{_body_file} if $self->{_body_file};
+  # Close the temporary body file.
+  undef $self->{_body_file};
+
+  # Assuming the "unlink" up in the body_write() function above worked,
+  # this code will not be run.  However, some systems will not allow
+  # an unlink to occur on an opened file.
   if ($self->{_filename} and -e $self->{_filename}) {
     unlink $self->{_filename} or $self->log(0, "Could not unlink ", $self->{_filename}, ": $!");
   }
